<?php

/**
 * The SWISH-E indexer. Calls the swish-e indexer commandline via exec.
 * Returns the standard output of swish-e.
 */
function _swish_do_index(){
  // Set up some paths to various directories
  $full_path = getcwd() . '/' . drupal_get_path('module','swish');
  $temp_file = tempnam(variable_get('file_directory_temp', '/tmp'), 'swish');
  // Change filepath if its private or public files
  if (variable_get('file_downloads','1') == FILE_DOWNLOADS_PUBLIC) {
    $file_path = getcwd() .'/' . file_directory_path();
  } else {
    $file_path = file_directory_path();
  }
  
  // Start making an "on the fly" configuration file
  $handle   = fopen($temp_file, 'w');
  fwrite($handle, "IncludeConfigFile $full_path/conf/common.conf\n"); // preset configuration stuff
  fwrite($handle,"IndexDir $file_path\n");
  fwrite($handle, "IgnoreWords file: $full_path/conf/stopwords/english.txt\n"); // ignored words
  fwrite($handle, "FileRules pathname contains stopwords CVS database images includes misc modules scripts themes tmp uploadimage\n"); // ignore certain paths
  fwrite($handle, "FileRules filename contains .php .inc .module .sql  index.\n"); // ignore certain filetypes

  // Create the item specific filter settings
  $indexonly = 'IndexOnly .txt';
  $filefilter = "\n";
  if (($filter   = variable_get('catdoc_path', '/usr/local/bin/catdoc')) ){
    $indexonly  .= ' .doc';
    $filefilter .= 'FileFilter .doc         '.$filter. ' "-s8859-1 -d8859-1 \'%p\'"'. "\n";
  }
  if (($filter  = variable_get('xls2csv_path', '/usr/local/bin/xls2csv')) ){
    $indexonly  .= ' .xls';
    $filefilter .= 'FileFilter .xls         '.$filter.' "\'%p\'"'. "\n";
  }
  if (($filter  = variable_get('pdftotext_path', '/usr/local/bin/pdftotext')) ){
    $indexonly  .= ' .pdf';
    $filefilter .= 'FileFilter .pdf         '.$filter.' "\'%p\' -"'. "\n";
  }
  if (($filter  = variable_get('ppthtml_path', '/usr/local/bin/ppthtml')) ){
    $indexonly  .= ' .ppt';
    $filefilter .= 'FileFilter .ppt         '.$filter.' "\'%p\' -"'. "\n";
  }
  if (($filter  = variable_get("unrtf_path","/usr/bin/unrtf")) ) {
    $indexonly  .= ' .rtf';
    $filefilter .= 'FileFilter .rtf         '.$filter.' "--html \'%p\'"'. "\n";
  }
  $indexonly .= "\n";
  $filefilter .= "\n";
  
  // Finish making the "ONE THE FLY" configuration file
  fwrite ($handle, $indexonly.$filefilter);	

  // Create the swish-e index command and run the command
  $swish_indx_cmd = variable_get('swish_path', '/usr/local/bin/swish-e');
  $swish_indx_cmd .= " -c $temp_file ";
  $swish_indx_cmd .= " -f $file_path/my_swish_index"; // save the index to the files directory
  exec (escapeshellcmd($swish_indx_cmd), $results, $rv);
  
  // Close out the file handler, delete the temp file
  fclose($handle);
  // unlink($temp_file);

  // Return the results of the search
  return $results;
}

function _swish_do_text_extract($filepath) {
  switch(substr($filepath,(strlen($filepath)-4),4)) {
    case ".doc":
      $filter = variable_get('catdoc_path', '/usr/local/bin/catdoc');
      $arg = '-s8859-1 -d8859-1'.' '.$filepath;
    break;

    case ".xls":
      $filter = variable_get('xls2csv_path', '/usr/local/bin/xls2csv');
    $arg = $filepath;
    break;

    case ".pdf":
      $filter = variable_get('pdftotext_path', '/usr/local/bin/pdftotext');
    $arg = $filepath. ' ' . '-';
    break;

    case ".ppt":
      $filter = variable_get('ppthtml_path', '/usr/local/bin/ppthtml');
    $arg = $filepath. ' ' . '-';
    break;

    case ".rtf":
      $filter = variable_get("unrtf_path","/usr/bin/unrtf");
    $arg = '--html ' . $filepath;
    break;

    default:
    $flter = false;
    break;
  }

  if ($filter) { 
    // Limit the Output to 80000 characters
    $output = substr(shell_exec($filter . ' ' . $arg . ' '),0,80000);
    // and check if search result is already in UTF-8 (is there a better way)
    if (strpos($output,"i?1/2",0) !== false ) { // "i?1/2" is ALT+159  --> is already in utf8
      return $output;
    } else {
      return utf8_encode($output);
    }
  }
}



/**
* Does an update of the Swish-E Fulltext Table
*/
function _swish_do_update() {
  // Find all files available to the system
  $file_result = db_query("SELECT * FROM {files}");
  while ($file = db_fetch_array($file_result)) {
    // Determine if full text information is already available
    if (!db_result(db_query("SELECT `fulltext` from {swish_fulltext} where filepath = '%s' and nid = %d",$file['filepath'],$file['nid']))) {
      // If not, get the full text information available
      $text = _swish_do_text_extract($file['filepath']);
      // Update the swish_fulltext index with the full text information
      if(db_result(db_query("SELECT nid FROM {swish_fulltext} WHERE filepath = '%s' AND nid = %d", $file['filepath'], $file['nid']))) {
        db_query("UPDATE {swish_fulltext} SET `fulltext` = '%s' WHERE `filepath` = '%s'",$text, $file['filepath']);
      } else {
        db_query("INSERT INTO {swish_fulltext} (filepath,nid,`fulltext`) VALUES ('%s',%d,'%s')",$file['filepath'],$file['nid'],$text);
      } 	  
    }
  }
}
