<?php

/**
* Implementation of hook_perm()
*/
function swish_perm(){
  return array('administer swish-e');
}

/**
* Implementation of hook_menu()
*/
function swish_menu($may_cache){
  $items = array();   
  if ($may_cache) {
    $items[] = array(
      'path'     => 'admin/settings/swish', 
      'description' => t('Settings to configure Swish-E and its various helper applications.'),
      'title'    => 'Swish-E',
      'callback' => 'drupal_get_form',
      'callback arguments' => 'swish_settings_form',
      'access'   => user_access('administer swish-e'), 
      'type' => MENU_NORMAL_ITEM,
    );  
  }
  return $items;
}

/**
* Implementation of hook_help
*/
function swish_help($section = 'admin/help#search') {	
  switch ($section) {
    case 'admin/help#swish':
      return t('This module allows the user of the search indexer Swish-E (http://www.swish-e.org). Many different file formats are supported (.doc, .xls, .txt, .ppt, .pdf, .rtf), but each file format filter must be specifically configured. Please download and install the newest version of Swish-E (http://www.swish-e.org) and any needed file filters and specify their filepaths on the configuration page.');
    case 'admin/settings/swish':
      return t('SWISH-E and Filters Binary Path Settings. Please note that only those file types that have working filters installed are to be indexed. So if you dont want a certain file type to be indexed, just leave the field for that file type filter blank. Text files dont need a filter, so it\'s indexed by default.');
	case 'admin/settings/swish/index':
	  return t('SWISH-E indexer. If no errors, you can start manually index the files here. Or let Drupal\'s cron do the job for you.');
  }	
}

/**
* The menu callback that creates the Swish-E settings form
*/
function swish_settings_form(){
  $form = array();
  
  // Form Stuff for Swish-E Filter Settings
  $form['file_filters'] = array(
    '#type' => 'fieldset',
    '#title' => 'Swish-E Filters',
    '#description' => t('In order to have the module successfully work, as many of the individual file filters as possible need to be enabled. If a document type does not have the correct file filter, it will not correctly pull down the full text information.'),
  );
  $form['file_filters']['swish_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Swish-E Path'),
    '#default_value' => variable_get('swish_path', '/usr/local/bin/swish-e'),
    '#size' => 60,
    '#maxlength' => 120,
    '#description' => t('The absolute path of the swish-e binary.'),
    '#required' => true,
    );
  $form['file_filters']['catdoc_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Microsoft Word Filter Path'),
    '#default_value' => variable_get('catdoc_path', '/usr/local/bin/catdoc'),
    '#size' => 60,
    '#maxlength' => 120,
    '#description' => t('The absolute path of the Microsoft Word filter.'),
  );
  $form['file_filters']['xls2csv_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Microsoft Excel Filter Path'),
    '#default_value' => variable_get('xls2csv_path', '/usr/local/bin/xls2csv'),
    '#size' => 60,
    '#maxlength' => 120,
    '#description' => t('The absolute path of the Microsoft Excel filter'),
  );
  $form['file_filters']['ppthtml_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Microsoft Powerpoint Filter Path'),
    '#default_value' => variable_get('ppthtml_path', '/usr/local/bin/ppthtml'),
    '#size' => 60,
    '#maxlength' => 120,
    '#description' => t('The absolute path of the Microsoft Excel filter'),
  );
  $form['file_filters']['pdftotext_path'] = array(
    '#type' => 'textfield',
    '#title' => t('PDF Filter Path'),
    '#default_value' => variable_get('pdftotext_path', '/usr/local/bin/pdftotext'),
    '#size' => 60,
    '#maxlength' => 120,
    '#description' => t('The absolute path of the PDF filter.'),
  );
  $form['file_filters']['unrtf_path'] = array(
    '#type' => 'textfield',
    '#title' => t('RTF Filter Path'),
    '#default_value' => variable_get('unrtf_path', '/usr/bin/unrtf'),
    '#size' => 60,
    '#maxlength' => 120,
    '#description' => t('The absolute path of the Rich Text Format filter.'),
  );  
  
  // Form Setup for Swish-E Indexing Setting
   $form['indexing'] = array(
    '#type' => 'fieldset',
    '#title' => 'Swish-E Indexing'
  ); 
  $form['indexing']['indexnow'] = array(
    '#type' => 'checkbox',
    '#title' => t('Begin Swish-E Indexing'),
  );    
  
 
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Settings'),
  );
  
  return $form;
}


/**
 *  The menu callback that validates the the Swish-E settings form
 */
function swish_settings_form_validate($form_id, $form_values){
  foreach($form_values as $variable => $value) {
    switch ($variable) {
    
      // Do Nothing for non file paths
      case "submit":
      case "form_id":
      case "form_token":
      case "indexnow":
      case "op":
      break;
      
      // Check to see if the file paths are valid
      default:
        if ($value && !is_executable($value)) {
          form_set_error($variable, $variable . ' ' . t('does not exist or is not executable'));
        }
      break;
    }
  }
}

function swish_settings_form_submit($form_id, $form_values){
  foreach($form_values as $variable => $value) {
    switch ($variable) {
    
      // Do nothing for values that are not file paths
      case "submit":
      case "form_id":
      case "form_token":
      case "indexnow":
      case "op":
      break;
      
      // Save the values for the values that are file paths
      default:
        variable_set($variable,$value);
      break;
    }
  }
  
  // Do the indexing if its necessary
  if ($form_values['indexnow'] <> 0) {
    $result = _swish_do_index();	
    if (substr($result[0], 0, 5) == 'err: ' ){
      drupal_set_message('Indexing aborted. '.$result[0], 'error');
    } else {
      $length = count($result);
      //include only the last 3 lines
      for ($i = $length - 1 ; $i >= ($length - 3) ; --$i)
        $msg .= $result[$i]."<br/>";
      drupal_set_message($msg);   
    }
  }
  drupal_set_message(t('Your Swish-E settings has been saved.'));
}
